@@ -19,7 +19,7 @@ import re
 from .utils import F
 from . import utils
 
-__version__ = '1.2.0'
+__version__ = '1.4.0'
 
 logger = logging.getLogger(__name__)
 
@@ -228,6 +228,18 @@ class CrossEnvBuilder(venv.EnvBuilder):
             raise FileNotFoundError("No _sysconfigdata*.py found in host lib")
 
 
+    def _is_python_source_dir(self, d):
+        fn = getattr(sysconfig, '_is_python_source_dir', None)
+        if fn:
+            return fn(d)
+
+        # removed in python 3.11
+        for fn in ("Setup", "Setup.local"):
+            if os.path.isfile(os.path.join(d, "Modules", fn)):
+                return True
+        return False
+
+
     def find_host_python(self, host):
         """
         Find Python paths and other info based on a path.
@@ -245,7 +257,7 @@ class CrossEnvBuilder(venv.EnvBuilder):
         else:
             self.host_project_base = os.path.dirname(host)
 
-        if sysconfig._is_python_source_dir(self.host_project_base):
+        if self._is_python_source_dir(self.host_project_base):
             self.host_makefile = os.path.join(self.host_project_base, 'Makefile')
             pybuilddir = os.path.join(self.host_project_base, 'pybuilddir.txt')
             try:
@@ -512,6 +524,16 @@ class CrossEnvBuilder(venv.EnvBuilder):
                 raise ValueError("Unexpected major version %s for MACOSX_DEPLOYMENT_TARGET" %
                         major)
 
+        if self.host_sysname == "darwin":
+            self.sysconfig_platform = "macosx-%s-%s" % (self.macosx_deployment_target,
+                self.host_machine)
+        elif self.host_sysname == "linux":
+            # Use self.host_machine here as powerpc64le gets converted
+            # to ppc64le in self.host_machine
+            self.sysconfig_platform = "linux-%s" % (self.host_machine)
+        else:
+            self.sysconfig_platform = self.host_platform
+
     def expand_manylinux_tags(self):
         """
         Convert legacy manylinux tags to PEP600, because pip only looks for one
@@ -686,6 +708,16 @@ class CrossEnvBuilder(venv.EnvBuilder):
         # directory.
         stdlib = os.path.abspath(os.path.dirname(os.__file__))
 
+        # In python 3.11, the import machinery imports from math, which breaks
+        # in our cross environment.. so we inject lib-dynload to the path also
+        dynload = os.path.join(stdlib, "lib-dynload")
+
+        # In python 3.11, several system packages are frozen by default, which
+        # prevents us from patching it. Disable it.
+        xopt = subprocess.run([context.build_env_exe, "--help-xoptions"],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        disable_frozen_modules = (b'frozen_modules=' in xopt.stdout)
+
         context.sentinel = random.randint(0,0xffffffff)
 
         extra_envs = list(self.extra_env_vars)
